# -*- coding: utf-8 -*-
from django.contrib import admin
from django.core.urlresolvers import reverse
import models
from forms import NavTypeForm, ArticleAdminForm, NewsletterItemAdminForm, NewsletterAdminForm
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from coop_cms.settings import get_article_class, get_navtree_class
from django.conf import settings
from settings import is_localized, can_rewrite_url

if 'modeltranslation' in settings.INSTALLED_APPS:
    from modeltranslation.admin import TranslationAdmin
    BaseAdminClass = TranslationAdmin
else:
    BaseAdminClass = admin.ModelAdmin


class NavNodeAdmin(admin.ModelAdmin):
    list_display = ["label", 'parent', 'ordering', 'in_navigation', 'content_type', 'object_id']

admin.site.register(models.NavNode, NavNodeAdmin)


class NavTypeAdmin(admin.ModelAdmin):
    form = NavTypeForm

admin.site.register(models.NavType, NavTypeAdmin)


class NavTreeAdmin(admin.ModelAdmin):
    list_display = ['__unicode__', 'name', 'navtypes_list']
    list_editable = ['name']
    list_filters = ['id']

    def nodes_li(self, tree):
        root_nodes = tree.get_root_nodes()
        nodes_li = u''.join([node.as_jstree() for node in root_nodes])
        return nodes_li

    def navtypes_list(self, tree):
        if tree.types.count() == 0:
            return _(u'All')
        else:
            return u' - '.join([unicode(x) for x in tree.types.all()])
    navtypes_list.short_description = _(u'navigable types')

    def change_view(self, request, object_id, extra_context=None):
        extra_context = extra_context or {}
        tree = models.get_navtree_class().objects.get(id=object_id)
        extra_context['navtree'] = tree
        extra_context['navtree_nodes'] = self.nodes_li(tree)
        return super(NavTreeAdmin, self).change_view(request, object_id,
            extra_context=extra_context)

admin.site.register(get_navtree_class(), NavTreeAdmin)


class ArticleAdmin(BaseAdminClass):
    form = ArticleAdminForm
    list_display = ['slug', 'title', 'category', 'template_name', 'publication', 'headline', 'in_newsletter', 'modified']
    list_editable = ['publication', 'headline', 'in_newsletter', 'category']
    readonly_fields = ['created', 'modified', 'is_homepage']
    list_filter = ['publication', 'headline', 'in_newsletter', 'sites', 'homepage_for_site', 'category', 'template']
    date_hierarchy = 'publication_date'
    fieldsets = (
        #(_('Navigation'), {'fields': ('navigation_parent',)}),
        (_(u'General'), {'fields': ('slug', 'title', 'subtitle', 'publication', )}),
        (_(u'Settings'), {'fields': ('sites', 'template', 'category', 'headline', 'is_homepage', 'logo', 'in_newsletter')}),
        (_(u'Advanced'), {'fields': ('publication_date', 'homepage_for_site', 'created', 'modified')}),
        (_(u'Content'), {'fields': ('content','summary',)}),
        (_(u'Debug'), {'fields': ('temp_logo',)}),
    )
    
    def get_form(self, request, obj=None, **kwargs):
        form = super(ArticleAdmin, self).get_form(request, obj, **kwargs)
        form.current_user = request.user
        return form
    
admin.site.register(get_article_class(), ArticleAdmin)

admin.site.register(models.Link)
admin.site.register(models.Document)
admin.site.register(models.Image)
admin.site.register(models.PieceOfHtml)
admin.site.register(models.NewsletterSending)
admin.site.register(models.FragmentType)
admin.site.register(models.FragmentFilter)

class FragmentAdmin(BaseAdminClass):
    pass
    list_display = ['name', 'position', 'type', 'filter', 'css_class']
    list_filter = ['type', 'filter', 'css_class']

admin.site.register(models.Fragment, FragmentAdmin)


class ArticleCategoryAdmin(admin.ModelAdmin):
    list_display = ['name', 'ordering', 'in_rss']
    list_editable = ['ordering', 'in_rss']
admin.site.register(models.ArticleCategory, ArticleCategoryAdmin)

class NewsletterItemAdmin(admin.ModelAdmin):
    form = NewsletterItemAdminForm
    list_display = ['content_type', 'content_object', 'ordering']
    list_editable = ['ordering']
    fieldsets = (
        (_('Article'), {'fields': ('object_id', 'content_type', 'ordering')}),
    )

admin.site.register(models.NewsletterItem, NewsletterItemAdmin)


class NewsletterAdmin(admin.ModelAdmin):
    form = NewsletterAdminForm
    raw_id_fields = ['items']
    list_display = ['subject', 'template', 'source_url']

    def get_form(self, request, obj=None, **kwargs):
        form = super(NewsletterAdmin, self).get_form(request, obj, **kwargs)
        form.current_user = request.user
        return form

admin.site.register(models.Newsletter, NewsletterAdmin)

class AliasAdmin(admin.ModelAdmin):
    list_display = ['path', 'redirect_url']
    list_editable = ['redirect_url']
admin.site.register(models.Alias, AliasAdmin)



# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.template.loader import get_template
from django.template import Context
from coop_cms.models import Link, Fragment
from coop_cms.settings import get_article_class, get_navtree_class
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from coop_bar.utils import make_link

def can_do(perm, object_names):
    def inner_decorator(func):
        def wrapper(request, context):
            editable = context.get('editable')
            if not editable:
                return
            for object_name in object_names:
                obj = context.get(object_name, None)
                if obj:
                    #print '>', func.__name__, object_name, obj
                
                    callback_name = u"coop_cms_{0}_callback".format(perm, object_name)
                    callback = context.get(callback_name, None)
                    
                    if callback and request and callback():
                        yes_we_can = func(request, context)
                        if yes_we_can:
                            return yes_we_can
                    
                #if object and request and request.user.has_perm(perm+"_"+object_name, object):
                #    yes_we_can = func(request, context)
                #    if yes_we_can:
                #        return yes_we_can
            return
        return wrapper
    return inner_decorator

can_edit_article = can_do('can_edit', ['article'])
can_edit_object = can_do('can_edit', ['article', 'object', 'objects'])
can_publish_article = can_do('can_publish', ['article'])
can_edit_newsletter = can_do('can_edit', ['newsletter'])
can_edit = can_do('can_edit', ['article', 'newsletter', 'object'])

def can_add_article(func):
    def wrapper(request, context):
        Article = get_article_class()
        ct = ContentType.objects.get_for_model(Article)
        perm = '{0}.add_{1}'.format(ct.app_label, ct.model)
        if request and request.user.has_perm(perm):
            return func(request, context)
        return None
    return wrapper

def can_add_link(func):
    def wrapper(request, context):
        ct = ContentType.objects.get_for_model(Link)
        perm = '{0}.add_{1}'.format(ct.app_label, ct.model)
        if request and request.user.has_perm(perm):
            return func(request, context)
        return None
    return wrapper

def django_admin(request, context):
    if request and request.user.is_staff:
        return make_link(reverse("admin:index"), _(u'Administration'), 'fugue/tables.png',
            classes=['icon', 'alert_on_click'])
        
def django_admin_edit_article(request, context):
    if request and request.user.is_staff and 'article' in context:
        article_class = get_article_class()
        article = context['article']
        view_name = 'admin:%s_%s_change' % (article_class._meta.app_label,  article_class._meta.module_name)
        return make_link(reverse(view_name, args=[article.id]), _(u'Article admin'), 'fugue/table.png',
            classes=['icon', 'alert_on_click'])

def django_admin_edit_object(request, context):
    if request and request.user.is_staff and 'object' in context:
        object = context['object']
        object_class = object.__class__
        view_name = 'admin:%s_%s_change' % (object_class._meta.app_label,  object_class._meta.module_name)
        try:
            return make_link(reverse(view_name, args=[object.id]),
                _(u'Admin {0}'.format(object_class._meta.verbose_name)), 'fugue/table.png',
                classes=['icon', 'alert_on_click'])
        except:
            pass

def django_admin_add_object(request, context):
    if request and request.user.is_staff and (('object' in context) or ('model' in context)):
        object_class = context.get('model', None)
        if not object_class:
            object_class = context['object'].__class__
        view_name = 'admin:%s_%s_add' % (object_class._meta.app_label,  object_class._meta.module_name)
        try:
            return make_link(reverse(view_name),
                _(u'Add {0}'.format(object_class._meta.verbose_name)), 'fugue/table.png',
                classes=['icon', 'alert_on_click'])
        except:
            pass

def django_admin_list_objects(request, context):
    if request and request.user.is_staff and (('object' in context) or ('model' in context)):
        object_class = context.get('model', None)
        if not object_class:
            object_class = context['object'].__class__
        try:
            view_name = 'admin:%s_%s_changelist' % (object_class._meta.app_label,  object_class._meta.module_name)
            return make_link(reverse(view_name),
                _(u'List {0}'.format(object_class._meta.verbose_name)), 'fugue/table.png',
                classes=['icon', 'alert_on_click'])
        except:
            pass


def django_admin_navtree(request, context):
    if request and request.user.is_staff:
        coop_cms_navtrees = context.get('coop_cms_navtrees', None)
        if coop_cms_navtrees:
            tree_class = get_navtree_class()
            admin_tree_name = "{0}_{1}".format(tree_class._meta.app_label, tree_class._meta.module_name)
            if len(coop_cms_navtrees) == 1:
                tree = coop_cms_navtrees[0]
                url = reverse('admin:{0}_change'.format(admin_tree_name), args=[tree.id])
                label = _(u'Navigation tree')
            else:
                url = reverse('admin:{0}_changelist'.format(admin_tree_name))
                label = _(u'Navigation trees')
            return make_link(url, label, 'fugue/leaf-plant.png',
                classes=['icon', 'alert_on_click'])

def view_all_articles(request, context):
    if request and request.user.is_staff:
        return make_link(reverse('coop_cms_view_all_articles'), _(u'Articles'), 'fugue/documents-stack.png',
            classes=['icon', 'alert_on_click'])

@can_edit
def cms_media_library(request, context):
    if context.get('edit_mode'):
        return make_link(reverse('coop_cms_media_images'), _(u'Media library'), 'fugue/images-stack.png',
            'coopbar_medialibrary', ['icon', 'slide'])
        
@can_edit
def cms_upload_image(request, context):
    if context.get('edit_mode'):
        return make_link(reverse('coop_cms_upload_image'), _(u'Add image'), 'fugue/image--plus.png',
            classes=['coopbar_addfile', 'colorbox-form', 'icon'])

@can_edit
def cms_upload_doc(request, context):
    if context.get('edit_mode'):
        return make_link(reverse('coop_cms_upload_doc'), _(u'Add document'), 'fugue/document-import.png',
            classes=['coopbar_addfile', 'colorbox-form', 'icon'])

@can_add_article
def cms_new_article(request, context):
    if not context.get('edit_mode'):
        url = reverse('coop_cms_new_article')
        return make_link(url, _(u'Add article'), 'fugue/document--plus.png',
            classes=['alert_on_click', 'colorbox-form', 'icon'])

@can_add_link
def cms_new_link(request, context):
    if not context.get('edit_mode'):
        url = reverse('coop_cms_new_link')
        return make_link(url, _(u'Add link'), 'fugue/document--plus.png',
            classes=['alert_on_click', 'colorbox-form', 'icon'])

@can_add_article
def cms_set_homepage(request, context):
    article = context.get('article', None)
    if context.get('edit_mode') and article and (not getattr(article, 'is_homepage', False)):
        url = reverse('coop_cms_set_homepage', args=[article.id])
        return make_link(url, _(u'Set homepage'), 'fugue/home--pencil.png',
            classes=['alert_on_click', 'colorbox-form', 'icon'])

@can_edit_article    
def cms_article_settings(request, context):
    #if context.get('edit_mode'):
    article = context['article']
    url = reverse('coop_cms_article_settings', args=[article.id])
    return make_link(url, _(u'Article settings'), 'fugue/gear.png',
        classes=['alert_on_click', 'colorbox-form', 'icon'])

@can_edit_object
def cms_save(request, context):
    if context.get('edit_mode'):
        #No link, will be managed by catching the js click event
        return make_link('', _(u'Save'), 'fugue/disk-black.png', id="coopbar_save",
            classes=['icon'])
    
def cms_save2(request, context):
    return
    if request and request.user.is_staff:
        url = context.get('coop_cms_edit_url', None)
        if url and context.get('edit_mode'):
            #No link, will be managed by catching the js click event
            return make_link('', _(u'Save'), 'fugue/disk-black.png', id="coopbar_save",
                classes=['icon'])

#@can_edit_article
#def cms_view(request, context):
#    if context.get('edit_mode'):
#        article = context['article']
#        return make_link(article.get_cancel_url(), _(u'View'), 'fugue/eye--arrow.png',
#            classes=['alert_on_click', 'icon', 'show-clean'])

@can_edit_object
def cms_cancel(request, context):
    if context.get('edit_mode'):
        url = context.get('coop_cms_cancel_url', None)
        if url:
            return make_link(url, _(u'Cancel'), 'fugue/cross.png', classes=['alert_on_click', 'icon'])
        
def cms_cancel2(request, context):
    return
    if request and request.user.is_staff:
        url = context.get('coop_cms_cancel_url', None)
        if url and context.get('edit_mode'):
            return make_link(url, _(u'Cancel'), 'fugue/cross.png',
                classes=['alert_on_click', 'icon'])
        
@can_edit_object
def cms_edit(request, context):
    if not context.get('edit_mode'):
        url = context.get('coop_cms_edit_url', None)
        if url:
            return make_link(url, _(u'Edit'), 'fugue/document--pencil.png', classes=['icon'])
        
def cms_edit2(request, context):
    return
    if request and request.user.is_staff:
        url = context.get('coop_cms_edit_url', None)
        if url and not context.get('edit_mode'):
            return make_link(url, _(u'Edit'), 'fugue/document--pencil.png', classes=['icon'])

@can_publish_article
def cms_publish(request, context):
    article = context.get('article')
    if article and ('draft' in context) :
        if context['draft']:
            
            return make_link(article.get_publish_url(), _(u'Draft'), 'fugue/lock.png',
                classes=['colorbox-form', 'icon'])
        else:
            return make_link(article.get_publish_url(), _(u'Published'), 'fugue/globe.png',
                classes=['colorbox-form', 'icon'])
        

def cms_extra_js(request, context):
    t = get_template("coop_cms/_coop_bar_js.html")
    return t.render(context)
    
def log_out(request, context):
    if request and request.user.is_authenticated() and request.user.is_staff:
        return make_link(reverse("django.contrib.auth.views.logout"), _(u'Log out'), 'fugue/control-power.png',
            classes=['alert_on_click', 'icon'])

@can_add_article
def cms_new_newsletter(request, context):
    if not context.get('edit_mode'):
        if getattr(settings, 'COOP_CMS_NEWSLETTER_TEMPLATES', None):
            url = reverse('coop_cms_new_newsletter')
            return make_link(url, _(u'Create newsletter'), 'fugue/document--plus.png',
                classes=['alert_on_click', 'colorbox-form', 'icon'])

@can_edit_newsletter
def edit_newsletter(request, context):
    if not context.get('edit_mode'):
        newsletter = context.get('newsletter')
        return make_link(newsletter.get_edit_url(), _(u'Edit'), 'fugue/document--pencil.png', classes=['icon'])

@can_edit_newsletter
def newsletter_admin(request, context):
    newsletter = context.get('newsletter')
    object_class = newsletter.__class__
    view_name = 'admin:%s_%s_change' % (object_class._meta.app_label,  object_class._meta.module_name)
    try:
        return make_link(reverse(view_name, args=[newsletter.id]),
            _(u'Admin {0}'.format(object_class._meta.verbose_name)), 'fugue/table.png',
            classes=['icon', 'alert_on_click'])
    except:
        pass

@can_edit_newsletter
def newsletter_articles(request, context):
    view_name = 'admin:coop_cms_newsletteritem_changelist'
    try:
        return make_link(reverse(view_name),
            _(u'Articles ordering'), 'fugue/table.png',
            classes=['icon', 'alert_on_click'])
    except:
        pass
    
@can_edit_newsletter
def cancel_edit_newsletter(request, context):
    if context.get('edit_mode'):
        newsletter = context.get('newsletter')
        return make_link(newsletter.get_absolute_url(), _(u'Cancel'), 'fugue/cross.png', classes=['icon'])

@can_edit_newsletter
def save_newsletter(request, context):
    newsletter = context.get('newsletter')
    post_url = context.get('post_url')
    if context.get('edit_mode') and post_url:
        return make_link(post_url, _(u'Save'), 'fugue/disk-black.png',
            classes=['icon', 'post-form'])

@can_edit_newsletter
def change_newsletter_settings(request, context):
    if not context.get('edit_mode'):
        newsletter = context.get('newsletter')
        url = reverse('coop_cms_newsletter_settings', kwargs={'newsletter_id': newsletter.id})
        return make_link(url, _(u'Newsletter settings'), 'fugue/gear.png',
            classes=['icon', 'colorbox-form', 'alert_on_click'])

#DEPRECATED
@can_edit_newsletter
def change_newsletter_template(request, context):
    if context.get('edit_mode'):
        newsletter = context.get('newsletter')
        url = reverse('coop_cms_change_newsletter_template', args=[newsletter.id])
        return make_link(url, _(u'Newsletter template'), 'fugue/application-blog.png',
            classes=['alert_on_click', 'colorbox-form', 'icon'])
###############

@can_edit_newsletter
def test_newsletter(request, context):
    newsletter = context.get('newsletter', None)
    if newsletter:
        url = reverse('coop_cms_test_newsletter', args=[newsletter.id])
        return make_link(url, _(u'Send test'), 'fugue/mail-at-sign.png',
            classes=['alert_on_click', 'colorbox-form', 'icon'])

#@can_edit_newsletter
#def schedule_newsletter(request, context):
#    if not context.get('edit_mode'):
#        newsletter = context.get('newsletter')
#        url = reverse('coop_cms_schedule_newsletter_sending', args=[newsletter.id])
#        return make_link(url, _(u'Schedule sending'), 'fugue/alarm-clock--arrow.png',
#            classes=['alert_on_click', 'colorbox-form', 'icon'])

def cms_add_fragment(request, context):
    if request:
        ct = ContentType.objects.get_for_model(Fragment)
        perm = '{0}.add_{1}'.format(ct.app_label, ct.model)
        if request.user.has_perm(perm):
            url = reverse("coop_cms_add_fragment")
            return make_link(url, _(u'Add fragment'), 'fugue/block--plus.png',
                    classes=['alert_on_click', 'colorbox-form', 'icon', 'if-fragments'])

def cms_edit_fragments(request, context):
    if request:
        ct = ContentType.objects.get_for_model(Fragment)
        perm = '{0}.change_{1}'.format(ct.app_label, ct.model)
        if request.user.has_perm(perm):
            url = reverse("coop_cms_edit_fragments")
            return make_link(url, _(u'Edit fragments'), 'fugue/block--pencil.png',
                    classes=['alert_on_click', 'colorbox-form', 'icon', 'if-fragments'])

def load_commands(coop_bar):
    
    coop_bar.register([
        [log_out],
        [django_admin, django_admin_edit_article, django_admin_edit_object, django_admin_navtree, view_all_articles],
        [cms_add_fragment, cms_edit_fragments],
        [cms_media_library, cms_upload_image, cms_upload_doc],
        [cms_new_newsletter, edit_newsletter, cancel_edit_newsletter, save_newsletter,
            change_newsletter_settings, newsletter_admin, newsletter_articles, 
            test_newsletter],
        [cms_edit, cms_save, cms_cancel, cms_edit2],
        [cms_new_article, cms_new_link, cms_article_settings, cms_set_homepage],
        [cms_publish],
    ])
    
    coop_bar.register_header(cms_extra_js)
    
    #def js_code(request, context):
    #    return """<script>
    #    $(function() {
    #        $("a.modal").each(function(idx, elt) {
    #            $(elt).modal({
    #                remote: $(elt).attr('href')
    #            });
    #        });
    #    });
    #    </script>"""
    #
    #coop_bar.register_header(js_code)



# -*- coding: utf-8 -*-

from coop_cms.settings import get_article_class
from coop_cms.shortcuts import get_article
from django import template
register = template.Library()
from django.utils.text import slugify
from coop_cms.utils import dehtml as do_dehtml
from bs4 import BeautifulSoup
import unicodedata
from django.conf import settings
from floppyforms import CheckboxInput
from coop_cms.models import ArticleCategory
import os.path

################################################################################
class ArticleLinkNode(template.Node):

    def __init__(self, title, lang):
        self.title = title
        self.lang = lang

    def render(self, context):
        Article = get_article_class()
        
        try:
            v = template.Variable(self.title)
            title = v.resolve(context)
        except template.VariableDoesNotExist:
            title = self.title.strip("'").strip('"')
        
        slug = slugify(title)
        try:
            if self.lang:
                article = get_article(slug, force_lang=self.lang)
            else:
                #If the language is not defined, we need to get it from the context
                #The get_language api doesn't work in templatetag
                request = context.get('request', None)
                lang = "en"
                if request:
                    lang = request.LANGUAGE_CODE
                elif hasattr(settings, 'LANGUAGES'):
                    lang = settings.LANGUAGES[0][0]
                elif hasattr(settings, 'LANGUAGE_CODE'):
                    lang = settings.LANGUAGE_CODE[:2]
                article = get_article(slug, current_lang=lang)
        except Article.DoesNotExist:
            try:
                article = get_article(slug, all_langs=True)
            except Article.DoesNotExist:
                article = Article.objects.create(slug=slug, title=title)
        return article.get_absolute_url()

@register.tag
def article_link(parser, token):
    args = token.split_contents()
    title = args[1]
    lang = args[2] if len(args) > 2 else None
    return ArticleLinkNode(title, lang)

@register.filter
def dehtml(value):
    return do_dehtml(value)
    
@register.filter
def sp_rt_lb(value):
    return value.replace("\n", " ").replace("\r", "")
    
################################################################################
class NewsletterFriendlyCssNode(template.Node):

    def __init__(self, nodelist_content, css):
        self.css = css
        self.nodelist_content = nodelist_content

    def render(self, context):
        content = self.nodelist_content.render(context)
        if context.get('by_email', False):
            soup = BeautifulSoup(content)
            for tag, css in self.css.items():
                for html_tag in soup.select(tag):
                    html_tag["style"] = css
            content = soup.prettify(formatter="minimal")
        else:
            style = ""
            for tag, value in self.css.items():
                style += u"{0} {{ {1} }}\n".format(tag, value)
            content = u"<style>\n{0}</style>\n".format(style) + content
        return content

@register.tag
def nlf_css(parser, token):
    #Newsletter friendly CSS
    args = token.split_contents()
    css = {}
    for item in args[1:]:
        tag, value = item.split("=")
        tag, value = tag.strip('"'), value.strip('"')
        css[tag] = value
    nodelist = parser.parse(('end_nlf_css',))
    token = parser.next_token()
    return NewsletterFriendlyCssNode(nodelist, css)

@register.filter
def normalize_utf8_to_ascii(ustr):
    try:
        return unicodedata.normalize('NFKD', ustr).encode('ascii','ignore')
    except TypeError:
        return ustr
    

@register.filter(name='is_checkbox')
def is_checkbox(field):
    field = getattr(field, 'field', field) # get the field attribute of the field or the field itself
    return field.widget.__class__.__name__ == CheckboxInput().__class__.__name__
    
@register.filter
def index(seq, index):
    try:
        return seq[index]
    except IndexError:
        return None

################################################################################
class CoopCategoryNode(template.Node):

    def __init__(self, cat_slug, var_name):
        cat = cat_slug.strip("'").strip('"')
        self.cat_var, self.cat = None, None
        if cat_slug == cat:
            self.cat_var = template.Variable(cat)
        else:
            self.cat = cat
        self.var_name = var_name

    def render(self, context):
        if self.cat_var:
            self.cat = self.cat_var.resolve(context)
        try:
            slug = slugify(self.cat)
            self.category = ArticleCategory.objects.get(slug=slug)
        except ArticleCategory.DoesNotExist:
            self.category = ArticleCategory.objects.create(name=self.cat)
        context.dicts[0][self.var_name] = self.category
        return ""

@register.tag
def coop_category(parser, token):
    #Newsletter friendly CSS
    args = token.split_contents()
    cat_slug = args[1]
    var_name = args[2]
    return CoopCategoryNode(cat_slug, var_name)


@register.filter
def basename(fullname):
    return os.path.basename(fullname)



from django import forms
from coop_cms.models import NavType, NavNode, Newsletter, NewsletterSending, Link, Document, Fragment, BaseArticle
from django.contrib.contenttypes.models import ContentType
from settings import get_navigable_content_types
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from djaloha.widgets import AlohaInput
import floppyforms
import re
from django.conf import settings
from coop_cms.settings import get_article_class, get_article_templates, get_newsletter_templates, get_navtree_class
from coop_cms.widgets import ImageEdit
from django.core.urlresolvers import reverse
from coop_cms.utils import dehtml
from datetime import datetime
from django.utils.timezone import now as dt_now
from coop_cms.widgets import ChosenSelectMultiple, ReadOnlyInput
from settings import is_localized, can_rewrite_url
from django.contrib.auth.models import Permission

class NavTypeForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(NavTypeForm, self).__init__(*args, **kwargs)
        self.fields['content_type'].widget = forms.Select(choices=get_navigable_content_types())

    def clean_label_rule(self):
        rule = self.cleaned_data['label_rule']
        if rule == NavType.LABEL_USE_GET_LABEL:
            ct = self.cleaned_data['content_type']
            if not 'get_label' in dir(ct.model_class()):
                raise ValidationError(_(u"Invalid rule for this content type: The object class doesn't have a get_label method"))
        return rule

    class Meta:
        model = NavType

class AlohaEditableModelForm(floppyforms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(AlohaEditableModelForm, self).__init__(*args, **kwargs)
        for field_name in self.Meta.fields:
            no_aloha_widgets = getattr(self.Meta, 'no_aloha_widgets', ())
            if not field_name in no_aloha_widgets: 
                self.fields[field_name].widget = AlohaInput()

    class Media:
        css = {
            'all': ('css/colorbox.css?v=2', ),
        }
        js = ('js/jquery.form.js', 'js/jquery.pageslide.js', 'js/jquery.colorbox-min.js?v=2', 'js/colorbox.coop.js?v=2')

class ArticleForm(AlohaEditableModelForm):

    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)
        self.article = kwargs.get('instance', None)
        self.set_logo_size()
        if getattr(settings, 'COOP_CMS_TITLE_OPTIONAL', False):
            #Optional title : make possible to remove the title from a template
            self.fields['title'].required = False

    class Meta:
        model = get_article_class()
        fields = ('title', 'subtitle', 'content', 'logo')
        no_aloha_widgets = ('logo',)

    def set_logo_size(self, logo_size=None, logo_crop=None):
        thumbnail_src = self.logo_thumbnail(logo_size, logo_crop)
        update_url = reverse('coop_cms_update_logo', args=[self.article.id])
        self.fields['logo'].widget = ImageEdit(update_url,
            thumbnail_src.url if thumbnail_src else '',
            attrs={"class": "resizable"})

    def logo_thumbnail(self, logo_size=None, logo_crop=None):
        if self.article:
            return self.article.logo_thumbnail(True, logo_size=logo_size, logo_crop=logo_crop)

    def clean_title(self):
        if getattr(settings, 'COOP_CMS_TITLE_OPTIONAL', False):
            title = self.cleaned_data['title']
            if not title and self.article:
                #if the title is optional and nothing is set
                #We do not modify it when saving
                return self.article.title
        else:
            title = self.cleaned_data['title'].strip()
            if title[-4:].lower() == '<br>':
                title = title[:-4]
            if not title:
                raise ValidationError(_(u"Title can not be empty"))
        return title


def get_node_choices():
    prefix = "--"
    #choices = [(None, _(u'<not in navigation>')), (0, _(u'<root node>'))]
    #for root_node in NavNode.objects.filter(parent__isnull=True).order_by('ordering'):
    #    for (progeny, level) in root_node.get_progeny():
    #        choices.append((progeny.id, prefix*level+progeny.label))
    #return choices
    choices = [(None, _(u'<not in navigation>'))]
    for tree in get_navtree_class().objects.all():
        choices.append((-tree.id, tree.name))
        for root_node in NavNode.objects.filter(tree=tree, parent__isnull=True).order_by('ordering'):
            for (progeny, level) in root_node.get_progeny():
                choices.append((progeny.id, prefix*(level+1)+progeny.label))
    return choices

def get_navigation_parent_help_text():
    return get_article_class().navigation_parent.__doc__

class NewsletterItemAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(NewsletterItemAdminForm, self).__init__(*args, **kwargs)
        self.item = kwargs.get('instance', None)
        article_choices = [(a.id, unicode(a)) for a in get_article_class().objects.all()]
        self.fields['object_id'] = forms.ChoiceField(
           choices=article_choices, required=True, help_text=_(u"Select an article")
        )
        self.fields['content_type'].required = False
        self.fields['content_type'].widget = forms.HiddenInput()

    def clean_content_type(self):
        return ContentType.objects.get_for_model(get_article_class())

class WithNavigationModelForm(forms.ModelForm):
    navigation_parent = forms.ChoiceField()
    
    def __init__(self, *args, **kwargs):
        super(WithNavigationModelForm, self).__init__(*args, **kwargs)
        #self.instance = kwargs.get('instance', None)
        self.fields['navigation_parent'] = forms.ChoiceField(
            choices=get_node_choices(), required=False, help_text=get_navigation_parent_help_text()
        )
        if self.instance:
            self.fields['navigation_parent'].initial = self.instance.navigation_parent

    def clean_navigation_parent(self):
        parent_id = self.cleaned_data['navigation_parent']
        parent_id = int(parent_id) if parent_id != 'None' else None
        return parent_id

    def save(self, commit=True):
        instance = super(WithNavigationModelForm, self).save(commit=False)
        parent_id = self.cleaned_data['navigation_parent']
        if instance.id:
            if instance.navigation_parent != parent_id:
                instance.navigation_parent = parent_id
        else:
            setattr(instance, '_navigation_parent', parent_id)
        if commit:
            instance.save()
        return instance

class ArticleAdminForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(ArticleAdminForm, self).__init__(*args, **kwargs)
        self.article = kwargs.get('instance', None)
        templates = get_article_templates(self.article, self.current_user)
        if templates:
            self.fields['template'].widget = forms.Select(choices=templates)
        
        Article = get_article_class()
        
        self.slug_fields = []
        if is_localized():
            for (lang, _name) in settings.LANGUAGES:
                self.slug_fields.append('slug_'+lang)
        else:
            self.slug_fields = ['slug']
        
        can_change_article_slug = can_rewrite_url()
        
        if not can_change_article_slug:
            can_change_article_slug = (self.article.publication != BaseArticle.PUBLISHED) if self.article else True
        
        for slug_field in self.slug_fields:
            if not can_change_article_slug:
                self.fields[slug_field].widget = ReadOnlyInput()

    class Meta:
        model = get_article_class()
        widgets = {
            'title': forms.TextInput(attrs={'size': 100})
        }

class AddImageForm(forms.Form):
    image = forms.ImageField(required=True, label = _('Image'),)
    descr = forms.CharField(required=False, widget=forms.TextInput(
        attrs={'size': '35', 'placeholder': _(u'Optional description'),}),
        label = _('Description'),
    )

class AddDocForm(forms.ModelForm):
    class Meta:
        model = Document
        fields = ('file', 'name', 'is_private', 'category')

class ArticleTemplateForm(forms.Form):
    def __init__(self, article, user, *args, **kwargs):
        super(ArticleTemplateForm, self).__init__(*args, **kwargs)
        choices = get_article_templates(article, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["template"].initial = article.template


class ArticleLogoForm(forms.Form):
    image = forms.ImageField(required=True, label = _('Logo'),)

class ArticleSettingsForm(WithNavigationModelForm):
    class Meta:
        model = get_article_class()
        fields = ('template', 'category', 'publication', 'publication_date', 'headline', 'in_newsletter', 'summary',)

    def __init__(self, user, *args, **kwargs):
        article = kwargs['instance']

        try:
            initials = kwargs['initial']
        except:
            initials = {}
        summary = article.summary
        if not summary:
            summary = dehtml(article.content)[:400]
        initials.update({'summary': summary})
        initials.update({'publication_date': article.publication_date.strftime("%Y-%m-%d %H:%M:%S")})
        
        kwargs['initial'] = initials
        super(ArticleSettingsForm, self).__init__(*args, **kwargs)

        self.fields['category'].queryset = self.fields['category'].queryset.filter(sites=settings.SITE_ID)

        choices = get_article_templates(article, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()

class NewArticleForm(WithNavigationModelForm):
    class Meta:
        model = get_article_class()
        fields = ('title', 'template', 'category', 'headline', 'publication', 'in_newsletter')

    def __init__(self, user, *args, **kwargs):
        super(NewArticleForm, self).__init__(*args, **kwargs)
        choices = get_article_templates(None, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["title"].widget = forms.TextInput(attrs={'size': 30})


class NewLinkForm(WithNavigationModelForm):
    
    class Meta:
        model = Link
        fields = ('title', 'url',)
    

class NewNewsletterForm(forms.ModelForm):
    class Meta:
        model = Newsletter
        fields = ('subject', 'template', 'items')
        #widgets = {}
        #try:
        #    widgets.update({
        #        'items': ChosenSelectMultiple(),
        #    })
        #except NameError:
        #    print 'NO ChosenSelectMultiple'
        #    pass

    def __init__(self, user, *args, **kwargs):
        super(NewNewsletterForm, self).__init__(*args, **kwargs)
        tpl_choices = get_newsletter_templates(None, user)
        if tpl_choices:
            self.fields["template"] = forms.ChoiceField(choices=tpl_choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["subject"].widget = forms.TextInput(attrs={'size': 30})


class PublishArticleForm(forms.ModelForm):
    class Meta:
        model = get_article_class()
        fields = ('publication',)# 'summary', 'category')
        widgets = {
            'publication': forms.HiddenInput(),
        }

    #def __init__(self, *args, **kwargs):
    #    article = kwargs['instance']
    #    try:
    #        initials = kwargs['initial']
    #    except:
    #        initials = {}
    #    summary = article.summary
    #    if not summary:
    #        summary = dehtml(article.content)[:250]
    #    initials.update({'summary': summary})
    #    kwargs['initial'] = initials
    #    super(PublishArticleForm, self).__init__(*args, **kwargs)

class NewsletterForm(AlohaEditableModelForm):
    class Meta:
        model = Newsletter
        fields = ('content',)
    
#class NewsletterForm(floppyforms.ModelForm):
#
#    class Meta:
#        model = Newsletter
#        fields = ('content',)
#        widgets = {
#            'content': AlohaInput(text_color_plugin=False),
#        }
#
#    class Media:
#        css = {
#            'all': ('css/colorbox.css', ),
#        }
#        js = ('js/jquery.form.js', 'js/jquery.pageslide.js', 'js/jquery.colorbox-min.js', 'js/colorbox.coop.js')
#

class NewsletterSchedulingForm(floppyforms.ModelForm):
    class Meta:
        model = NewsletterSending
        fields = ('scheduling_dt',)

    def clean_scheduling_dt(self):
        sch_dt = self.cleaned_data['scheduling_dt']

        if not sch_dt:
            raise ValidationError(_(u"This field is required"))

        if sch_dt < dt_now():
            raise ValidationError(_(u"The scheduling date must be in future"))

        return sch_dt

class NewsletterTemplateForm(forms.Form):

    def __init__(self, newsletter, user, *args, **kwargs):
        super(NewsletterTemplateForm, self).__init__(*args, **kwargs)
        choices = get_newsletter_templates(newsletter, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["template"].initial = newsletter.template
        
        

class NewsletterAdminForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(NewsletterAdminForm, self).__init__(*args, **kwargs)
        self.newsletter = kwargs.get('instance', None)
        choices = get_newsletter_templates(self.newsletter, self.current_user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()

    class Meta:
        model = Newsletter
        fields = ('subject', 'content', 'template', 'source_url', 'items')
        widgets = {}
        try:
            widgets.update({
                'items': ChosenSelectMultiple(),
            })
        except NameError:
            #print 'No ChosenSelectMultiple'
            pass

    class Media:
        css = {
            'all': ('css/admin-tricks.css',),
        }
        js = ()
        
class AddFragmentForm(forms.ModelForm):

    class Meta:
        model = Fragment
        fields = ('type', 'name', 'position', 'filter')
        
    def __init__(self, data=None, *args, **kwargs):
        super(AddFragmentForm, self).__init__(data, *args, **kwargs)
        self.fields['filter'].widget = forms.HiddenInput()
        
class EditFragmentForm(forms.ModelForm):
    delete_me = forms.BooleanField(label=_(u"delete"), required=False)

    class Meta:
        model = Fragment
        fields = ('type', 'filter', 'name', 'css_class', 'position')
        widgets = {
            "type": forms.HiddenInput(),
            "filter": forms.HiddenInput(),
        }
        
    def __init__(self, *args, **kwargs):
        super(EditFragmentForm, self).__init__(*args, **kwargs)
        choices = [('', '')]+[(x, x) for x in self.instance.type.allowed_css_classes.split(',')]
        self.fields['css_class'].widget = forms.Select(choices=choices)
        
    def clean_css_class(self):
        val = self.cleaned_data['css_class']
        if val:
            if not val in self.instance.type.allowed_css_classes.split(','):
                val = u""
        return val
        
    def save(self, *args, **kwargs):
        if self.cleaned_data['delete_me']:
            self.instance.delete()
            return None
        return super(EditFragmentForm, self).save(*args, **kwargs)
    